/*
 * lilJudd
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.lilJudd

import com.kotlindiscord.kord.extensions.ExtensibleBot
import dev.kord.common.entity.PresenceStatus
import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.launch
import net.moonleay.botendo.build.BuildConstants
import net.moonleay.lilJudd.data.CredentialManager
import net.moonleay.lilJudd.data.DB
import net.moonleay.lilJudd.extensions.FeatureManageExtension
import net.moonleay.lilJudd.extensions.VersionExtension
import net.moonleay.lilJudd.features.TimePlanner
import net.moonleay.lilJudd.util.Logger
import kotlin.system.exitProcess

object Bot {
    //The kord object gets set at app launch
    lateinit var bot: ExtensibleBot
    suspend fun start() {
        Logger.out("Starting Bot...")

        //Load config
        CredentialManager.load()

        //Don't run the bot when there is no bot token in config
        if (CredentialManager.token == "empty") {
            Logger.out("The config does not contain a bot token.")
            exitProcess(3)
        }

        //Check if the credentials for the Database are existent
        if (CredentialManager.dbDomain == "empty" || CredentialManager.dbName == "empty" || CredentialManager.dbUser == "empty" || CredentialManager.dbPassword == "empty") {
            Logger.out("The config does not contain the whole Database credentials.")
            exitProcess(3)
        }

        //Connect to the database
        DB.connect(
            CredentialManager.dbDomain,
            CredentialManager.dbName,
            CredentialManager.dbUser,
            CredentialManager.dbPassword
        )

        //Register the TimePlanner thread
        val coroutineJob = GlobalScope.launch {
            TimePlanner.registerThread()
        }

        // Add a shutdown hook to cancel the coroutine when the application is terminated
        Runtime.getRuntime().addShutdownHook(Thread {
            coroutineJob.cancel()
        })


        //Add Bot token to kord
        bot = ExtensibleBot(CredentialManager.token) {
            applicationCommands {
                enabled = true
            }

            extensions {
                add(::VersionExtension)
                add(::FeatureManageExtension)
            }

            this.presence {
                this.status = PresenceStatus.DoNotDisturb
                this.playing("v." + BuildConstants.version)
            }
        }

        //Start the bot
        bot.start()
    }
}
