/*
 * lilJudd
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.lilJudd.features

import com.kotlindiscord.kord.extensions.utils.addReaction
import dev.inmo.krontab.buildSchedule
import dev.inmo.krontab.doInfinity
import dev.kord.common.Color
import dev.kord.common.entity.Snowflake
import dev.kord.core.behavior.channel.createMessage
import dev.kord.core.entity.Message
import dev.kord.core.entity.channel.MessageChannel
import kotlinx.coroutines.delay
import net.moonleay.lilJudd.Bot
import net.moonleay.lilJudd.data.tables.TimePlanningChannels
import net.moonleay.lilJudd.util.Logger
import net.moonleay.lilJudd.util.MessageUtil
import org.jetbrains.exposed.sql.selectAll
import org.jetbrains.exposed.sql.transactions.transaction
import java.time.ZoneId
import java.time.ZonedDateTime


object TimePlanner {
    /*  /--------------- Seconds
        | /------------- Minutes
        | | /----------- Hours
        | | | /--------- Days of months
        | | | | /------- Months
        | | | | | /----- (optional) Year
        | | | | | | /--- (optional) Timezone offset
        | | | | | | |  / (optional) Week days
        * * * * * * 0o *w*/

    suspend fun registerThread() {
        Logger.out("Adding scheduler...")
        val scheduler = buildSchedule("0 0 4 * * * 0o 1") // 0 0 4 * * * 0o 1 // 0o is UTC
        scheduler.doInfinity {
            Logger.out("Starting to notify...")
            val channelList = mutableListOf<Snowflake>()
            transaction {
                for (tp in TimePlanningChannels.selectAll()) {
                    channelList.add(Snowflake(tp[TimePlanningChannels.channelid]))
                    Logger.out("Have to notify channel with ID ${tp[TimePlanningChannels.channelid]}.")
                }
            }
            Logger.out("${channelList.count()} Channels to notify!")
            for (ch in channelList) {
                if (Bot.bot.kordRef.getChannel(ch)!!.asChannelOrNull() == null)
                    return@doInfinity
                val c = Bot.bot.kordRef.getChannelOf<MessageChannel>(ch)!!
                c.createMessage {
                    this.embeds.add(
                        MessageUtil.getEmbed(
                            Color(0X4C4645),
                            "Time Planning Feature",
                            "Do you have time on the following Days?",
                            "Automated Message"
                        )
                    )
                }
                delay(3000)
                var then = ZonedDateTime.now(ZoneId.of("Europe/Berlin")).withHour(4).withMinute(0).withSecond(0)
                repeat(6) {
                    val m = c.createMessage {
                        this.embeds.add(
                            MessageUtil.getEmbedSmall(
                                Color(0X4C4645),
                                "",
                                "${then.dayOfWeek.name}, ${then.dayOfMonth}.${then.monthValue}.${then.year} /${it + 1}. weekday"
                            )
                        )
                    }
                    then = then.plusDays(1).withHour(20).withMinute(24).withSecond(0)
                    delay(500)
                    addReactions(m)
                    delay(1000)
                    Logger.out("Finished sending day $it")
                }
                Logger.out("Finished with ${c.data.guildId.value}")
            }
            Logger.out("Done! Until next Monday! <3   ")
        }
    }

    private suspend fun addReactions(msg: Message) {
        msg.addReaction("✅")
        delay(300)
        msg.addReaction("❌")
        delay(300)
        msg.addReaction("❓")
        delay(300)
    }
}
